# This module should calculate verification metrics at any time of the workflow
# It should implement different verification metrics:
# - FRPSS and RPSS
# - FCRPSS abd CRPSS
# - enscorr
# - bias
# - reliability diagram
# - ask Carlos which decadal metrics he is currently using

source("modules/Skill/s2s.metrics.R")
## TODO: Remove once the new version of s2dv is released
source("https://earth.bsc.es/gitlab/es/s2dv/-/raw/master/R/RandomWalkTest.R")
source("https://earth.bsc.es/gitlab/es/s2dv/-/raw/develop-RPSS/R/RPS.R")
source("https://earth.bsc.es/gitlab/es/s2dv/-/raw/develop-RPSS/R/RPSS.R")

## TODO: Implement this in the future
## Which parameter are required?
# if (!("obs" %in% ls()) || is.null(obs)) {
#   error(logger,
#         "There is no object 'obs' in the global environment or it is NULL")
# }
# if (stream == "fcst" && (!("fcst" %in% ls()) || is.null(fcst))) {
#   error(logger,
#         "There is no object 'fcst' in the global environment or it is NULL")
# }
# if (!("hcst" %in% ls()) || is.null(hcst)) {
#   error(logger,
#         "There is no object 'hcst' in the global environment or it is NULL")
# }
# if (!("metric" %in% ls()) || is.null(metric)) {
#   warn(logger,
#        "Verification metric not found and it is set as 'EnsCorr'.")
#   metric <- 'EnsCorr'
# }
# if (metric %in% c('FRPSS', 'RPSS')) {
#   metric_fun <- "veriApply"
#   metric_method <- "FairRpss"
# } else if (metric %in% c("FCRPSS", "CRPSS")) {
#   metric_fun <- "veriApply"
# } else if (metric %in% c("EnsCorr", "EnsCor")) {
#   metric_fun <- "veriApply"
#   metric_method <- "EnsCorr"
# #...
# } else {
#   error(logger, "Unknown verification metric defined in the recipe.")
#   metric_fun <- 'NotFound'
# }
# info(logger, paste("#-------------------------- ", "\n",
#                    " running Skill module ", "\n",
#                    " it can call ", metric_fun ))

compute_skill_metrics <- function(exp, obs, recipe, na.rm = T, ncores = 1) {
  # exp: s2dv_cube containing the hindcast
  # obs: s2dv_cube containing the observations
  # recipe: auto-s2s recipe as provided by read_yaml
  
  ## TODO: Adapt time_dims to subseasonal case?
  time_dim <- 'syear'
  memb_dim <- 'ensemble'
  metric <- recipe$Analysis$Workflow$Skill$metric
  # Whether the fair version of the metric is to be computed
  if (metric %in% c('FRPS', 'FRPSS', 'BSS10', 'BSS90')) {
    Fair <- T
  } else {
    Fair <- F
  }

  if (metric %in% c('RPS', 'FRPS')) {
    skill <- RPS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
		 Fair = Fair, ncores = ncores)

  } else if (metric %in% c('RPSS', 'FRPSS')) {
    skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
		  Fair = Fair, ncores = ncores)
  } else if (metric == 'BSS10') {
    skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
		  prob_thresholds = 0.1, Fair = Fair, ncores = ncores)
  } else if (metric == 'BSS90') {
    skill <- RPSS(exp$data, obs$data, time_dim = time_dim, memb_dim = memb_dim,
		  prob_thresholds = 0.9, Fair = Fair, ncores = ncores)
  }

  return(skill)

}

  ## TODO: Add specsVerification funs

